
/* omap3_target.S
 *
 * (C) 2008, Nishanth Menon <menon.nishanth@gmail.com>
 * (C) 2008, Dirk Behme, dirk.behme@gmail.com
 *
 * This small test application is loaded into internal
 * OMAP3 SRAM and there blinks with user LEDs. I.e. after
 * downloading via OMAP3 serial or USB boot ROM you should be able
 * see blinking user LEDs.
 *
 * ----------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *---------------------------------------------------------------------------*/

/****************** DEFINES ************************/
/* Work around HW errata on BeagleBoards rev A */
//#define BEAGLE_REVB

/* Pin mux definitions */
#define DIS		(0 << 3)
#define EN		(1 << 3)
#define PTD		(0 << 4)
#define IDIS		(0 << 8)
#define IEN		(1 << 8)
#define M0		0
#define M4		4

/* WDT Regs */
#define WDT_WWPS 	0x34	/* Write posted status */
#define WDT_WSPR 	0x48	/* Start/Stop reg */

/* GPIO Regs */
#define GPIO_CTRL	0x30
#define GPIO_OE		0x34
#define GPIO_DATAO	0x3C
#define GPIO_SET	0x94
#define GPIO_CLR	0x90

#define GPIO_ENABLE	0x01


/****************** MAIN CODE ************************/
.text
.globl _start
_start:
	/* Reset mode to SVC mode */
	mrs r0, cpsr
	bic r0, r0, #0x1F
	orr r0, r0, #0x13
	orr r0, r0, #0xC0
	msr cpsr, r0

	/* Kill WatchDog Timer
	 * There are 3 watch dogs WD1=Secure, WD2=MPU, WD3=IVA. WD1 is
	 * either taken care of by ROM (HS/EMU) or not accessible (GP).
	 * We need to take care of WD2-MPU or take a PRCM reset.  WD3
	 * should not be running and does not generate a PRCM reset.
	 */
	ldr r0, WD2_BASE

	ldr r1, WD_UNLOCK1
	str r1, [r0, #WDT_WSPR]
	nop

wait1:
	ldr r2, [r0, #WDT_WWPS]
	cmp r2, #0
	bne wait1

	ldr r1, WD_UNLOCK2
	str r1, [r0, #WDT_WSPR]
	nop

wait2:
	ldr r2, [r0, #WDT_WWPS]
	cmp r2, #0
	bne wait2


	/* Enable the I/F clock for 32Khz clk. */
	ldr r1, EN_CLK32
	ldr r0, CM_ICLKEN_WKUP
	ldr r2, [r0]
	orr r2, r2, r1
	str r2, [r0]


	/* LED Control Logic (Start) */
	/*
	 * There are 2 LEDs on Beagle
	 * USR0=D6=LED0_GPIO150 on Ball W8 - uart1_cts = 0x48002180(LOW)
	 * USR1=D7=LED1_GPIO149 on Ball AA9 - uart1_rts= 0x4800217C(HIGH)
	 * Need to do two things:
	 * a) pin mux it to the right point
	 * b) Enable I/F clks to GPIO, Enable GPIO,
	 * Enable Pin Mux Pull high to force a glow
	 * Do a half word access to update 16 bit part of the reg
	 */
	ldr  r1, PADCONF_GPIO_MODE4
	ldr  r0, CONTROL_PADCONF_UART1_TX_HIGH
	strh r1, [r0]
	ldr  r0, CONTROL_PADCONF_UART1_CTS_LOW
	strh r1, [r0]

	/* Both GPIO 149 and 150 fall into GPIO5 module.
	 * Enable I and F Clocks for GPIO5 */
	ldr r1, EN_GPIO5
	ldr r0, CM_FCLKEN_PER
	ldr r2, [r0]
	orr r2, r2, r1
	str r2, [r0]
	ldr r0, CM_ICLKEN_PER
	ldr r2, [r0]
	orr r2, r2, r1
	str r2, [r0]
	/* Setup Blinky LEDs */
	ldr r0, OMAP34XX_GPIO5_BASE

	/* Enable the Module First */
	ldr r1, [r0, #GPIO_CTRL]
	orr r1, r1, #GPIO_ENABLE /* set to enable the Module */
	str r1, [r0, #GPIO_CTRL]

	ldr r2, GPIO_PATTERN1
	ldr r3, GPIO_PATTERN2

	ldr  r1, [r0, #GPIO_OE]
	/* Clear the pattern bits to enable output */
	bic r1, r1, r2
	bic r1, r1, r3
	str r1, [r0, #GPIO_OE]

blink:
	/* Enable R2 pattern ON */
	str r2, [r0, #GPIO_DATAO]

	/* delay */
	ldr r4, DELAY_COUNT
	bl  delay

	/* Rotate the pattern values */
	mov r1, r3
	mov r3, r2
	mov r2, r1

	/* Infinite loop */
	b blink
	/* LED Control Logic (End) */

/****** Common Function *******/
delay:
	/* Uses: R5-R8
	 * Delay value: r4
	 */
	ldr r5, CLK32K_COUNT_REGISTER
	ldr r7, [r5] /* Base value to handle overflow */

delay_loop:
	/* Check for the timer register overflow */
	ldr r6, [r5]
	/* if R6(current val) is >=R7(old value), continue */
	cmp r6, r7
	bhs delay_continue
	/* RollOver - Reset values */
	/* Get how far was r7 from max */
	ldr r8, DELAY_MAX
	sub r7, r8, r7
	/* Subtract time passed by from r4 */
	sub r4, r4, r7
	/* Reset R7 to Roll over start */
	mov r7, #0

delay_continue:
	/* get the time expired since base time */
	sub r6, r6, r7
	/* Check if time is the delay time */
	cmp r4, r6
	/* if R4(Delay) greater than R6(current time exipred),
	   then loop again */
	bhs delay_loop /* more to go.. repeat */

	/* Hooray!! we are done.. return */
	mov  pc, lr
	/* Spacing */
	nop
	nop
	nop

/****************** DATA AREA ************************/
/* WDT */
WD2_BASE:			.word 0x48314000
WD_UNLOCK1:			.word 0xAAAA
WD_UNLOCK2:			.word 0x5555

/* Pin Mux */
CONTROL_PADCONF_UART1_TX_HIGH:	.word (0x4800217C + 0x2)
CONTROL_PADCONF_UART1_CTS_LOW:	.word 0x48002180
/* Mode 4, Pull DIS	*/
PADCONF_GPIO_MODE4:		.word (M4)

/* Clock registers */
CM_FCLKEN_PER:			.word 0x48005000
CM_ICLKEN_PER:			.word 0x48005010
CM_ICLKEN_WKUP:			.word 0x48004C10
EN_GPIO5:			.word (0x1 << 16)
EN_CLK32:			.word (0x1 << 2)

/* GPIO regs */
OMAP34XX_GPIO5_BASE:		.word 0x49056000

#ifdef BEAGLE_REVB
/* FOR REVB boards flip each led */
GPIO_PATTERN1:			.word (0x1 << 22) /* GPIO 150 */
GPIO_PATTERN2:			.word (0x1 << 21) /* GPIO 149 */
#else /* REVA */
GPIO_PATTERN1:			.word ((0x1 << 21) | (0x1 << 22))
GPIO_PATTERN2:			.word (0x0)
#endif

/* Counter register */
CLK32K_COUNT_REGISTER:		.word 0x48320010
DELAY_COUNT:			.word 0x00008000 /* 32768 = 1sec */
DELAY_MAX:			.word 0xFFFFFFFF
